/************************************************************************

 Module:      JTVRemote.cpp

 Project:     JTV

 Description: FlyVideo Generic RCU driver v1.0.
              

 Copyright (c) 2002 by Ivan Kolin  <ivkolin@mailru.com>

 All rights reserved.

 You may not use any piece of the following code for commercial purposes
 without written permission from the author.

 You may freely copy and distribute the source code as long as it contains
 this copyright notice and the disclaimer. You may freely modify it.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.    IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
#include <unknwn.h>
#include <stdio.h>

#include "jtvrem.h"
#include "jtvbt8x8.h"
#include "btregs.h"
#include "jtvremsk.h"

// This driver corresponds to the JTVRemote Specification 1.2.
// Look at JTVRemSpec1x.zip file for additional information.

// Key codes from: Patrick Vicol, <vpatrickd@yahoo.com>,

// Firm    : FlyVideo
// Kit Type: Retail
// Mark    : FlyVideo
// Model   : FlyVideo 98 TV Capturer (LR50 rev.S)
// Tuner   : Philips
// Standart: PAL
// System  : D/K

// BTTool.exe display for "FlyVideo 98 (LR50 rev.S)":
// GPIO_Dma_Ctl - 0000c004; (0000c007 when TV window open)
// GPIO_Out_En  - 00001800;
// GPIO_Data    - remarked column (for pressed - released
//                keys and loaded/unload4ed JTV).
// When open JTV-window, GPIO_Data values can change it's
// values in regular way that can be used while driver creation.
// Example: 00xxxxxx - 00xxxxxc, where "x" - random value.
// GPIO_Data    - 0084e300 (when JTV-window is open)
// GPIO_Data    - 0084ff00 (when JTV-window is closed)
//
// When JTV is started, GPIO_Data turns to 0084e300 (when no key pressed on remote) and
// 0084e2xx when a key is pressed. After quitting JTV, GPIO_Data shows 0084ff00 and when
// a key is pressed, it shows 0084fexx
// When driver's tv program is started, GPIO_Data shows 0094e300 (when no key is pressed)
// and 0094e2xx when a key on the remote is pressed. After closing the tv app, the
// GPIO_Data shows 0094ff00 and when a key is pressed, it shows 0094fexx
// Basically, driver's tv application turns on bit4 of byte yy from GPIO_Data
// with the convention that GPIO_Data is wwxxyyzz, and bits are 7-0 (hi-lo), while JTV
// turns bit4 off

KeyDef keys[]=
{
//------[ table for driver creation ]---[GPIO_Data key codes source table to fill]
//                                                   JTV is loaded         JTV is NOT loaded
// 1-st row keys                                     Pressed    Released   Pressed  Released
        { 0x00, "Key_Power", "Power", },             //0084e200 - 0084e300 / 0084fe00 - 0084ff00
// No key
        { 0x80, "Key_MouseMode", "", },     //0084e280 - 0084e300 / 0084fe80 - 0084ff00
        { 0x40, "Key_FullScreen", "",},//0084e240 - 0084e300 / 0084fe40 - 0084ff00
// 2-nd row keys
        { 0x48, "Key_ChannelUp","ChannelUp",},       //0084e248 - 0084e300 / 0084fe48 - 0084ff00 (*)
        { 0xc0, "Key_1", "1", },                     //0084e2c0 - 0084e300 / 0084fec0 - 0084ff00
        { 0x20, "Key_2", "2", },                     //0084e220 - 0084e300 / 0084fe20 - 0084ff00
        { 0xa0, "Key_3", "3", },                     //0084e2a0 - 0084e300 / 0084fea0 - 0084ff00
// ...
        { 0xc8, "Key_ChannelDown","ChannelDown",},   //0084e2c8 - 0084e300 / 0084fec8 - 0084ff00 (**)
        { 0xe0, "Key_4", "4", },                     //0084e2e0 - 0084e300 / 0084fee0 - 0084ff00
        { 0x10, "Key_5", "5", },                     //0084e210 - 0084e300 / 0084fe10 - 0084ff00
        { 0x90, "Key_6", "6", },                     //0084e290 - 0084e300 / 0084fe90 - 0084ff00

        { 0x28, "Key_VolumeUp","VolumeUp",},         //0084e228 - 0084e300 / 0084fe28 - 0084ff00
        { 0xd0, "Key_7", "7", },                     //0084e2d0 - 0084e300 / 0084fed0 - 0084ff00
        { 0x30, "Key_8", "8", },                     //0084e230 - 0084e300 / 0084fe30 - 0084ff00
        { 0xb0, "Key_9", "9", },                     //0084e2b0 - 0084e300 / 0084feb0 - 0084ff00

        { 0xe8, "Key_VolumeDown","VolumeDown", },    //0084e2e8 - 0084e300 / 0084fee8 - 0084ff00
        { 0xf8, "Key_Cancel", "", },           //0084e2f8 - 0084e300 / 0084fef8 - 0084ff00
        { 0xf0, "Key_0", "0", },                     //0084e2f0 - 0084e300 / 0084fef0 - 0084ff00
        { 0x08, "Key_Enter", "Enter", },             //0084e208 - 0084e300 / 0084fe08 - 0084ff00

        { 0x60, "Key_Recall","Recall",},             //0084e260 - 0084e300 / 0084fe60 - 0084ff00
        { 0x70, "Key_AirCableTV", "Up", },   //0084e270 - 0084e300 / 0084fe70 - 0084ff00
        { 0x88, "Key_Video", "", },             //0084e288 - 0084e300 / 0084fe88 - 0084ff00
        { 0x50, "Key_Inc", "NextLinear", },                 //0084e250 - 0084e300 / 0084fe50 - 0084ff00

//        { 0x48, "Key_Left", "", },               //0084e248 - 0084e300 / 0084fe48 - 0084ff00 (*)
//        { 0xc8, "Key_Empty", "", },             //0084e2c8 - 0084e300 / 0084fec8 - 0084ff00 (**)
        { 0xa8, "Key_Audio", "", },             //0084e2a8 - 0084e300 / 0084fea8 - 0084ff00
        { 0x68, "Key_Dec", "PrevLinear", },                 //0084e268 - 0084e300 / 0084fe68 - 0084ff00

        { 0xb8, "Key_Window", "", },           //0084e2b8 - 0084e300 / 0084feb8 - 0084ff00
        { 0x98, "Key_Down", "Down", },               //0084e298 - 0084e300 / 0084fe98 - 0084ff00
        { 0x58, "Key_Esc", "", },                 //0084e258 - 0084e300 / 0084fe58 - 0084ff00
// No key

        { 0x38, "Key_Right", "Right", },       //0084e238 - 0084e300 / 0084fe38 - 0084ff00
        { 0x78, "Key_Left", "Left", },         //0084e278 - 0084e300 / 0084fe78 - 0084ff00
        { 0xd8, "Key_Mute", "", },               //0084e2d8 - 0084e300 / 0084fed8 - 0084ff00
        { 0x18, "Key_AVSource", "", },       //0084e218 - 0084e300 / 0084fe18 - 0084ff00
};

// Please, mark with ">>" sign correct answer below or add your own.
// When any key is pressed (and keep pressing), GPIO_Data value:
// >> 1. Strongly keeps the pressed value
//    2. In random intervals (=> ??? ms) changes to value - ???
//    3. In interval ??? ms change to value 0x?? and back
//    4. Other case ???


// (*), (**), ... - duplicated codes!

typedef UINT (CALLBACK* LPFNDLLFUNC1)(DWORD,UINT);


DWORD dwParam1;
UINT  uParam2, uReturnVal;

INT first_delay=1000;	// default first delay for a key
INT repeat_delay=100;	// default repeat delay for a key

void log(const char* str, ...)
{
  va_list arglist;

  FILE* l=fopen("JTVRemote.log","a+t");
  if (l==NULL) return;
  va_start( arglist, str );
  vfprintf(l, str, arglist );
  va_end( arglist );
  fclose(l);

// Look at section [JTVRemote] in "jtv.ini" file, look for strings
// "KeyFirstDelay=" and "KeyRepeatDelay=", example:
//      [JTVRemote]
//      ............
//      KeyFirstDelay=800
//      KeyRepeatDelay=60
// If strings exist, get corresponded values into "first_delay"
// and "repeat_delay" variables, if not - keep default values above.

  first_delay=GetPrivateProfileInt("JTVRemote","KeyFirstDelay",first_delay,".\\jtv.ini");
  repeat_delay=GetPrivateProfileInt("JTVRemote","KeyRepeatDelay",repeat_delay,".\\jtv.ini");

}

class JTVRemote : public CRemoteDriver
{

public:

        JTVRemote(KeyDef *k, int count) : CRemoteDriver(k, count)
		{
			log("> Initializing FlyVideo Generic RCU driver v1.0.\n");
            log("> Copyright (c) 2002 by Ivan Kolin, <ivkolin@mailru.com>.\n");
			log("> KeyFirstDelay=%d, KeyRepeatDelay=%d\n",first_delay,repeat_delay);
		};


        void PollingThread()
        {
                IBT8x8Device *bt;
                DWORD last_code=0xffffffff;;
                DWORD last_tick=GetTickCount();
   
                if (SUCCEEDED(device->QueryInterface(IID_IBT8x8Device, (void **)&bt)))
                {
                    DWORD value;
                    INT key_pause=20; // new key initial delay
					DWORD press_flag=1;
					//DWORD test_code=0;

                    bt->ReadBT(BT848_GPIO_DMA_CTL, &value);
                    bt->WriteBT(BT848_GPIO_DMA_CTL, value | 0xC004);

                    bt->ReadBT(BT848_GPIO_OUT_EN, &value);
                    bt->WriteBT(BT848_GPIO_OUT_EN, (value | 0x00001800));

                    bt->ReadBT(BT848_GPIO_DATA, &value);
                    bt->WriteBT(BT848_GPIO_DATA, value | 0x94e300);
                   
				  	   
                    while (WaitForSingleObject(hBreakEvent, 0) != WAIT_OBJECT_0)
                    {
                         Sleep(key_pause);

                         DWORD gpio_data=0;
                         DWORD value=0;
						 
                         bt->ReadBT(BT848_GPIO_DATA, &gpio_data);
 						 
                         value = (gpio_data &0x000000ff);
					  	 press_flag = ((gpio_data & 0x00000100)>>8);
						 
						 // Test block begin - comment all while build checked out driver
						 //if (test_code != value)
						 //{
							//log(">> ---- Test information ---- \n");
							//log(">>      Tick_Counter: %d\n",GetTickCount());
							//log(">>         GPIO_DATA: %x\n",gpio_data);
							//log(">>  Current_Key_Code: %x\n",value);
							//log(">> Previous_Key_Code: %x\n",last_code);

							//log(">>    Press_Key_Flag: %x\n",press_flag);
							//test_code = value;
						 //}
           				 // Test block end

                         if (press_flag == 0) // any key is pressed
                         {	 
                             if (last_code == value)
                             {     // auto repeat for any pressed key
								   //log(">>    Press_Key_Auto: %x\n",value); //for test
                            	   key_pause = repeat_delay;
                             } 
                             else
                             {	   // first run for any pressed key
								   //log(">>   Press_Key_First: %x\n",value); //for test
                                   last_code = value;
                                   key_pause = first_delay;
                             }
                             AddStroke(last_code);                             
                         }
                         else  // no one key is pressed
                         {	  
                             if ((value!=last_code) && (last_code!=0xffffffff))
                             {
                                last_code = value;
                             }
							   
                                key_pause = 20;
                         };
				
                    };

                log("FlyVideo Generic: Break event received. Bye!\n");

                bt->Release();
                };
        };
};

extern "C"
{
        __declspec(dllexport) IJTVRemoteControl *__stdcall GetInterfacePtr()
        {
                return new JTVRemote(keys, sizeof(keys) / sizeof(keys[0]));
        };
};


